import xlsxwriter
import subprocess
import os
import shelve
import sys
import shutil
import re
import os.path
from PIL import Image
from uncompressed import uncompressed
import numpy as np
import random
from numba import njit, prange
import glob
import cv2

#  Creation of an excel file to store all the results

workbook = xlsxwriter.Workbook('Compression_shell.xlsx')

#  Initialization

Datas = ['Eucrib_03', 'Gama_01', 'Z275', 'Archie_04', 'C7', 'Probe_7', 'Pseudoskorpion', 'Screwjoint']
Temps = []
Meths = ['tar.gz', 'tar.bz2']
Blocks = [8, 16, 32]


# Function used to get the size of a folder

def get_size(start_path='.'):
    total_size = 0
    for dirpath, dirnames, filenames in os.walk(start_path):
        for f in filenames:
            fp = os.path.join(dirpath, f)
            if not os.path.islink(fp):
                total_size += os.path.getsize(fp)

    return total_size


# Function for checking if uncompressed data matched compressed data in terms of pixels


@njit(parallel=True)
def check(image1, image2):
    h, w = image1.shape[0], image1.shape[1]
    pix = 0

    for a in prange(h):
        for b in prange(w):
            if image1[a, b] == image2[a, b]:
                pix += 1
    return pix


try:
    os.mkdir('/data/source')
    os.mkdir('/data/pics')
    os.mkdir('/data/pics_vti')
except:
    print()

try:
    os.mkdir('/data/compressed')
    os.mkdir('/data/uncompressed')
except:
    print()

for k in Datas:

    # Fetching width, height and the number of pics of the current dataset

    im = Image.open('/data/' + str(k) + '/slices_8bit/' + str(
        random.choice(os.listdir('/data/' + str(k) + '/slices_8bit/'))))
    width, height = im.size
    DIR = '/data/' + str(k) + '/slices_8bit/'
    nb_pics = len([name for name in os.listdir(DIR) if os.path.isfile(os.path.join(DIR, name))])

    # Initialization

    worksheet = workbook.add_worksheet()
    Ratio = []
    Comp = []
    Temps = []

    for i in Blocks:

        shutil.rmtree('/data/source')
        shutil.rmtree('/data/pics')
        shutil.rmtree('/data/compressed')

        os.mkdir('/data/source')
        os.mkdir('/data/pics')
        os.mkdir('/data/compressed')
        os.mkdir('/data/compressed/Webp')
        os.mkdir('/data/compressed/Szip')
        os.mkdir('/data/compressed/Jpls')
        os.mkdir('/data/compressed/Jp2')
        os.mkdir('/data/compressed/Png')
        os.mkdir('/data/compressed/Tar.bz2')
        os.mkdir('/data/compressed/Tar.gz')

        bashCommand = "bash compress_full.sh " + str(i) + ' ' + str(k)
        print(bashCommand)
        ratio = [str(i)]

        process = subprocess.Popen(bashCommand.split(), stdout=subprocess.PIPE)
        output, error = process.communicate()
        output = str(output)

        # Executing decompression program and fetching decompression time

        temps = ['Decompress time (s)']
        temps.extend(uncompressed(width, height, nb_pics, i))

        # Fetching sizes of different folders in order to calculate compression ratio

        index_or = output.find('Original folder size:')
        size = output[index_or + 20:index_or + 30]
        size = str(re.findall("\d+\.\d+", size)).strip('[]')
        size_or = re.sub("[^\d\.]", '', size)
        if size_or == '':
            size_or = int(re.search(r'\d+', output[index_or + 20:index_or + 30]).group())

        print('size_or', size_or)

        cpt0 = 1
        comp = ['Lossless ?']

        for n in Meths:
            index = (output.find(str(n) + ' size:'))
            if n in ['jp2', 'szip', 'webp', 'jpls', 'png']:
                index = (output.find(str(n) + ' folder size:'))
            size = output[index + 7:index + 30]
            size = str(re.findall("\d+\.\d+", size)).strip('[]')
            size = re.sub("[^\d\.]", '', size)
            if size == '':
                size = int(re.search(r'\d+', output[index + 7:index + 30]).group())

            print('size', size)
            ratio.append(round(float(size_or) / float(size), 4))
            cpt0 += 1

            # Testing if compression was lossless using check function

            files1 = glob.glob('/data/source/*.tif')
            files1.sort(key=lambda x: int(''.join(filter(str.isdigit, x))))

            if i == 0:
                files2 = glob.glob('/data/uncompressed/bin/' + n + '/*.tif')
            else:
                files2 = glob.glob('/data/uncompressed/' + n + '/*.tif')

            files2.sort(key=lambda x: int(''.join(filter(str.isdigit, x))))

            num = 0
            state = []

            while num < len(files1):

                image1 = cv2.imread(files1[num], 0)
                image2 = cv2.imread(files2[num], 0)

                h, w = image1.shape[0], image1.shape[1]
                pix = check(image1, image2)

                if pix == h * w:
                    state.append('Yes')

                else:
                    state.append('No')

                num += 1

            if 'No' in state:
                comp.append('No')

            elif not state:
                comp.append('Error')

            else:
                comp.append('Yes')

        Ratio.append(ratio)
        Comp.append(comp)
        Temps.append(temps)

    Methods = ['tar.gz', 'tar.bz2']
    Methods.insert(0, str(k))
    Data = [Methods]
    for h in range(len(Blocks)):
        Data.append(Ratio[h])
        Data.append(Comp[h])
        Data.append(Temps[h])

    #  Filling rows and columns of the excel file with results

    for i in range(0, 3 * len(Blocks) + 1):
        for j in range(0, len(Methods)):
            worksheet.write(j, i, Data[i][j])
            worksheet.set_column(i, i, len(str('Decompress time (s)')))

workbook.close()
