# !/usr/bin/python

import tiledb
import numpy as np
from PIL import Image
import glob
import cv2
import sys
import shutil
import os
import xlsxwriter
import matplotlib.pyplot as plt
from os import walk

# Fetching different parameters

Datas = sys.argv[1]
Method = sys.argv[2]
tiles = sys.argv[3]

# Looking for corresponding data set

files = glob.glob('/data/'+str(Datas)+'/slices_8bit/*.tif')

files.sort(key=lambda x: int(''.join(filter(str.isdigit, x))))

# Creation of the 3D array

X_data = []

for myFile in files:

    img = cv2.imread(myFile, 0)
    X_data.append(img)

X_data = np.array(X_data)
size1, size2, size3 = X_data.shape[0], X_data.shape[1], X_data.shape[2]


# Initialization of tiledb domain

if tiles == '0':
    dom = tiledb.Domain(tiledb.Dim(name="rows", domain=(0, size2 - 1), tile=size2, dtype=np.int32),
                    tiledb.Dim(name="cols", domain=(0, size3 - 1), tile=size3, dtype=np.int32),
                    tiledb.Dim(name="numbers", domain=(0, size1 - 1), tile=size1, dtype=np.int32))

else:
    dom = tiledb.Domain(tiledb.Dim(name="rows", domain=(0, size2 - 1), tile=tiles, dtype=np.int32),
                    tiledb.Dim(name="cols", domain=(0, size3 - 1), tile=tiles, dtype=np.int32),
                    tiledb.Dim(name="numbers", domain=(0, size1 - 1), tile=tiles, dtype=np.int32))

if Method == 'Gzip':
    compression = tiledb.GzipFilter(level=9)

if Method == 'Bzip2':
    compression = tiledb.Bzip2Filter(level=9)

if Method == 'Zstd':
    compression = tiledb.ZstdFilter(level=9)

if Method == 'LZ4':
    compression = tiledb.LZ4Filter(level=9)

a_filters = tiledb.FilterList([compression])

schema = tiledb.ArraySchema(domain=dom, sparse=False,
                            attrs=[tiledb.Attr(name="a", dtype=np.int8, filters=a_filters)])

array_name = "quickstart_dense"
tiledb.DenseArray.create(array_name, schema)


# The 3D array is stored as a tiledb array

with tiledb.DenseArray(array_name, mode='w') as A:
    tiledb.stats_enable()
    A[:] = X_data
    tiledb.stats_dump()
    tiledb.stats_disable()

shutil.rmtree('/quickstart_dense')
os.mkdir('/quickstart_dense')
